/* QACPI - A window manager-independent applet for reading ACPI battery
          information on Compaq 1700 series (and perhaps other) computers
   Copyright (C) 2002  Daniel A. Wagenaar
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   
   Contacting the author:
   
      Daniel Wagenaar
      Caltech 103-33
      Pasadena, CA 91125
      USA
      wagenaar@caltech.edu
*/
// Battery.C

#include "Battery.H"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

Battery::Battery() {
  now=0;
  update();
}

Battery::BState Battery::getBState() {
  BState s;
  s.usage_mW=0;
  s.reserve_mWh=0;
  FILE *f = fopen(DEVICE,"r");
  if (!f) {
    perror("Battery: Cannot open device");
    return s;
  }
  char buf[100];
  bool inuse=false;
  while (!feof(f)) {
    char *b=fgets(buf,100,f);
    if (b) {
      char *key = strtok(buf,":");
      if (!key)
	continue;
      char *value = strtok(0,":");
      while (value && *value<=32)
	value++;
      value=strtok(value," \n");
      //      fprintf(stderr,"key=[%s] value=[%s]\n",key,value);
      if (strcmp(key,"present rate")==0)
	s.usage_mW = atoi(value);
      else if (strcmp(key,"charging state")==0)
	inuse = strcmp(value,"discharging")==0;
      else if (strcmp(key,"remaining capacity")==0)
	s.reserve_mWh = atoi(value);
    }
  }
  fclose(f);
  if (!inuse)
    s.usage_mW = 0;
  return s;
}

void Battery::update() {
  now++;
  if (now>=READINGS)
    now=0;
  state[now] = getBState();
  sumUsage_mW=0;
  nave=0;
  for (int k=0; k<READINGS; k++) {
    if (state[k].usage_mW>0) {
      sumUsage_mW += state[k].usage_mW;
      nave++;
    }
  }
  used = nave>0 && state[now].usage_mW>0;
  if (used) {
    int u=sumUsage_mW/nave;
    min = u ? (60*getReserve_mWh()/u) : 300;
  } else {
    min = 300;
  }
}
