/* QACPI - A window manager-independent applet for reading ACPI battery
          information on Compaq 1700 series (and perhaps other) computers
   Copyright (C) 2002  Daniel A. Wagenaar
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   
   Contacting the author:
   
      Daniel Wagenaar
      Caltech 103-33
      Pasadena, CA 91125
      USA
      wagenaar@caltech.edu
*/
#include "QACPI.H"
#include "QSegmentBar.H"

#define USE_MINUTES 0

#if USE_MINUTES
static const int qacpi_segments[] = { 60, 30, 15, 5 };
#else
static const int qacpi_segments[] = { 1000, 250, 100, 25 };
#endif
static const int qacpi_colors[][3] = { { 0,200,0}, {255,255,0}, {255,128,0}, {255,0,0} };

/* 
 *  Constructs a QACPI which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
QACPI::QACPI( QWidget* parent,  const char* name, WFlags fl )
    : QACPIBase( parent, name, fl )
{
  remainingtime->setRange(1000000); // dummy (mWh)
  currentusage->setRange(1800); // mW
  currentusage->setSegment(500); // mW
}

/*  
 *  Destroys the object and frees any allocated resources
 */
QACPI::~QACPI()
{
    // no need to delete child widgets, Qt does it all for us
}

void QACPI::timerEvent(QTimerEvent *qte) {
  battery.update();
  compute();
}

void QACPI::polish() {
  currentusage->setPaletteForegroundColor(QColor(0,128,255));
  currentusage->setPaletteBackgroundColor(QColor("grey75"));
  remainingtime->setPaletteBackgroundColor(QColor("grey75"));
  compute();
  startTimer(UPDATEIVAL_S * 1000);
}

void QACPI::compute() {
  int usage_mW = battery.getMeanUsage_mW();
#if USE_MINUTES
  int remaining = battery.getTime_min();
#else
  int remaining = battery.getReserve_mWh();
#endif
  int k=4;
  int range=0;
  int segment=0;
  while (k>0 && remaining>range) {
    --k;
    range=qacpi_segments[k]*4;
  }
  segment=qacpi_segments[k];
  QColor fg(qacpi_colors[k][0],qacpi_colors[k][1],qacpi_colors[k][2]);

  while (remaining > range)
    range+=segment;

  if (remainingtime->getRange() != range) {
    remainingtime->setRange(range);
    remainingtime->setSegment(segment);
    remainingtime->setPaletteForegroundColor(fg);
  }
  if (remainingtime->getValue() != remaining) {
    remainingtime->setValue(remaining);
  }
  if (currentusage->getValue() != usage_mW) {
    currentusage->setValue(usage_mW);
  }
}
