/* QACPI - A window manager-independent applet for reading ACPI battery
          information on Compaq 1700 series (and perhaps other) computers
   Copyright (C) 2002  Daniel A. Wagenaar
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   
   Contacting the author:
   
      Daniel Wagenaar
      Caltech 103-33
      Pasadena, CA 91125
      USA
      wagenaar@caltech.edu
*/
// QSegmentBar.C

#include "QSegmentBar.H"
#include <qpainter.h>

QSegmentBar::QSegmentBar(QWidget *parent, const char *name,
			 int range0, int segment0, WFlags f):
  QFrame(parent, name, f),
  range(range0), segment(segment0), value(0) {
  setFrameStyle(QFrame::Panel + QFrame::Sunken);
  setLineWidth(1);
}

void QSegmentBar::setRange(int range0) { range=range0; update(); }

void QSegmentBar::setSegment(int segment0) { segment=segment0; update(); }

void QSegmentBar::setValue(int value0) { value=value0; update(); }

void QSegmentBar::drawContents(QPainter *p) {
  // some sanity requirements:
  if (segment<2)
    segment=2;
  if (range<segment)
    range=segment;
  if (value<0)
    value=0;
  if (value>range)
    value=range;

  int nseg = value/segment;
  int dxseg = width()*segment/range;
  int spc = dxseg/30; if (spc<1) spc=1;
  int h = height();
  if (dxseg<3)
    nseg=0;
  QBrush b(foregroundColor());
  for (int i=0; i<nseg; i++)
    p->fillRect(i*dxseg, 0, dxseg-spc, h, b);
  p->fillRect(nseg*dxseg, 0, width()*(value - nseg*segment)/range, h, b);
}
